/* IoT Monitoring via Google Firebase
 * Nanobot Workshop (Instagram https://www.instagram.com/nanobot.workshop/ )
 * Android App (Free Version) https://play.google.com/store/apps/details?id=com.nanobot.iotmonitorfree
 * Android App (Pro Version) https://play.google.com/store/apps/details?id=com.nanobot.iotmonitorpro
 */

//Library
#include "FirebaseESP8266.h"
#include <ESP8266WiFi.h>
#include "DHT.h"

//Firebase Setting
#define FIREBASE_URL "FIREBASE URL"    //Firebase URL
#define FIREBASE_AUTH "FIREBASE AUTH"  //Firebase Token
#define WIFI_SSID "WIFI SSID"          //WiFi Name
#define WIFI_PASSWORD "WIFI PASSWORD"  //WiFi Password             

FirebaseData firebaseData;
FirebaseJson json;

//Pin Setting
#define sensorDHT D1    //Pin D1 --> Output Sensor
#define analogPin A0
#define trigPin D5
#define echoPin D6

//Variable Setting
#define DHTTYPE DHT11
DHT dht(sensorDHT, DHTTYPE);
float humidity;   //Humidity
float temp;       //Temperature
float analogValue = 0;
long duration;
float distance;

void setup() {
  Serial.begin(9600);
//Pin Mode
  pinMode(analogPin, INPUT);

//WiFi Configuration
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  Serial.print("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(300);
  }
  Serial.println();
  Serial.print("Connect");
  Serial.println(WiFi.localIP());
  Serial.println();

//Firebase Configuration
  Firebase.begin(FIREBASE_URL, FIREBASE_AUTH);
  Firebase.reconnectWiFi(true);

//DHT Configuration
  dht.begin();

//Ultrasonik Configuration
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

}

void readDHT(){
  humidity = dht.readHumidity();
  temp = dht.readTemperature();
  Serial.print("Humidity = ");
  Serial.print(humidity);
  Serial.print(" %");
  Serial.print("    ");
  Serial.print("Temperature : ");
  Serial.print(temp);
  Serial.println(" C");
}

void readAnalog(){
  analogValue = analogRead(analogPin);
  Serial.println(analogValue);
}

void ultrasonic(){
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);

  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);
  distance = duration * 0.034/2;

  Serial.print(distance);
  Serial.println(" cm");
}

void loop() {
  readDHT();
  if(Firebase.setFloat(firebaseData, "/SENSOR/humiDHT11", humidity)){
    Serial.println("HUMIDITY PASSED");
  }
  else{
    Serial.println("HUMIDITY FAILED BECAUSE " + firebaseData.errorReason());
  }
  
  if(Firebase.setFloat(firebaseData, "/SENSOR/tempDHT11", temp)){
    Serial.println("TEMPERATURE PASSED");
    Serial.println();
  }
  else{
    Serial.println("TEMPERATURE FAILED BECAUSE " + firebaseData.errorReason());
    Serial.println();
  }

  readAnalog();
  if(Firebase.setFloat(firebaseData, "/SENSOR/analog", analogValue)){
    Serial.println("ANALOG VALUE PASSED");
    Serial.println();
  }
  else{
    Serial.println("ANALOG VALUE FAILED BECAUSE " + firebaseData.errorReason());
    Serial.println();
  }

  ultrasonic();
  if(Firebase.setFloat(firebaseData, "/SENSOR/ultrasonic", distance)){
    Serial.println("DISTANCE PASSED");
    Serial.println();
  }
  else{
    Serial.println("DISTANCE FAILED BECAUSE " + firebaseData.errorReason());
    Serial.println();      
  }
}
